@@ -1,34 +1,41 @@
 """
-This file defines cache, session, and translator T object for the app
-These are fixtures that every app needs so probably you will not be editing this file
+This module sets up core fixtures and utilities for a py4web application, including:
+
+- Logging: Configures a custom logger using application settings.
+- Database: Connects to the database using settings from the configuration.
+- Caching: Instantiates a cache object for use throughout the app.
+- Translation: Sets up the translation object for internationalization.
+- Session Management: Selects and configures the session backend (cookies, Redis, Memcache, or database) based on settings.
+- Authentication: Initializes the Auth object, configures its parameters, and defines authentication tables and actions.
+- Email: Configures the email sender for authentication-related emails if SMTP settings are provided.
+- User Groups: Sets up tagging for user groups if the authentication database is available.
+- Auth Plugins: Optionally registers authentication plugins (PAM, LDAP, OAuth2 for Google, GitHub, Facebook, Okta) based on settings.
+- File Download: Defines an action for downloading uploaded files if an upload folder is specified.
+- Scheduler: Optionally starts a background scheduler for running tasks if enabled in settings.
+- Decorators: Provides convenience action factories for authenticated and unauthenticated routes.
+
+This file is intended to be a foundational part of the application and typically does not require modification.
 """
+
 import os
 import sys
-import logging
-from py4web import Session, Cache, Translator, Flash, DAL, Field, action
-from py4web.utils.mailer import Mailer
+
+from pydal.tools.scheduler import Scheduler
+from pydal.tools.tags import Tags
+
+from py4web import DAL, Cache, Field, Flash, Session, Translator, action
+from py4web.server_adapters.logging_utils import make_logger
 from py4web.utils.auth import Auth
 from py4web.utils.downloader import downloader
-from pydal.tools.tags import Tags
 from py4web.utils.factories import ActionFactory
+from py4web.utils.mailer import Mailer
+
 from . import settings
 
 # #######################################################
 # implement custom loggers form settings.LOGGERS
 # #######################################################
-logger = logging.getLogger("py4web:" + settings.APP_NAME)
-formatter = logging.Formatter(
-    "%(asctime)s - %(levelname)s - %(filename)s:%(lineno)d - %(message)s"
-)
-for item in settings.LOGGERS:
-    level, filename = item.split(":", 1)
-    if filename in ("stdout", "stderr"):
-        handler = logging.StreamHandler(getattr(sys, filename))
-    else:
-        handler = logging.FileHandler(filename)
-    handler.setFormatter(formatter)
-    logger.setLevel(getattr(logging, level.upper(), "DEBUG"))
-    logger.addHandler(handler)
+logger = make_logger("py4web:" + settings.APP_NAME, settings.LOGGERS)
 
 # #######################################################
 # connect to db
@@ -52,8 +59,9 @@ T = Translator(settings.T_FOLDER)
 # #######################################################
 if settings.SESSION_TYPE == "cookies":
     session = Session(secret=settings.SESSION_SECRET_KEY)
+
 elif settings.SESSION_TYPE == "redis":
-    import redis
+    import redis  # type: ignore[reportMissingImports]
 
     host, port = settings.REDIS_SERVER.split(":")
     # for more options: https://github.com/andymccurdy/redis-py/blob/master/redis/client.py
@@ -64,11 +72,15 @@ elif settings.SESSION_TYPE == "redis":
         else cs(k, v, e)
     )
     session = Session(secret=settings.SESSION_SECRET_KEY, storage=conn)
+
 elif settings.SESSION_TYPE == "memcache":
-    import memcache, time
+    import time
+
+    import memcache  # type: ignore[reportMissingImports]
 
     conn = memcache.Client(settings.MEMCACHE_CLIENTS, debug=0)
     session = Session(secret=settings.SESSION_SECRET_KEY, storage=conn)
+
 elif settings.SESSION_TYPE == "database":
     from py4web.utils.dbstore import DBStore
 
@@ -84,11 +96,12 @@ auth.param.registration_requires_approval = settings.REQUIRES_APPROVAL
 auth.param.login_after_registration = settings.LOGIN_AFTER_REGISTRATION
 auth.param.allowed_actions = settings.ALLOWED_ACTIONS
 auth.param.login_expiration_time = 3600
-auth.param.password_complexity = {"entropy": 50}
+auth.param.password_complexity = {"entropy": settings.PASSWORD_ENTROPY}
 auth.param.block_previous_password_num = 3
 auth.param.default_login_enabled = settings.DEFAULT_LOGIN_ENABLED
 auth.define_tables()
 auth.fix_actions()
+auth.logger = logger
 
 flash = auth.flash
 
@@ -134,6 +147,19 @@ if settings.OAUTH2GOOGLE_CLIENT_ID:
         )
     )
 
+if settings.OAUTH2GOOGLE_SCOPED_CREDENTIALS_FILE:
+    from py4web.utils.auth_plugins.oauth2google_scoped import (
+        OAuth2GoogleScoped,
+    )  # TESTED
+
+    auth.register_plugin(
+        OAuth2GoogleScoped(
+            secrets_file=settings.OAUTH2GOOGLE_SCOPED_CREDENTIALS_FILE,
+            scopes=[],  # Put here any scopes you want in addition to login
+            db=db,  # Needed to store credentials in auth_credentials
+        )
+    )
+
 if settings.OAUTH2GITHUB_CLIENT_ID:
     from py4web.utils.auth_plugins.oauth2github import OAuth2Github  # TESTED
 
@@ -167,15 +193,33 @@ if settings.OAUTH2OKTA_CLIENT_ID:
         )
     )
 
+# #######################################################
+# Enable optional API token plugins
+# #######################################################
+
+# curl -H "Authorization: Bearer {token}"
+# create tokens in db.auth_simple_token
+#
+# simple_token_plugin = SimpleTokenPlugin(auth)
+# auth.token_plugins.append(simple_token_plugin)
+
+# curl -H "Authorization: Bearer {token}"
+# create tokens with JwtTokenPlugin(auth).make(user, expiration)
+#
+# jwt_token_plugin = JwtTokenPlugin(auth)
+# auth.token_plugins.append(jwt_token_plugin)
+
 # #######################################################
 # Define a convenience action to allow users to download
 # files uploaded and reference by Field(type='upload')
 # #######################################################
 if settings.UPLOAD_FOLDER:
-    @action('download/<filename>')                                                   
-    @action.uses(db)                                                                                           
+
+    @action("download/<filename>")
+    @action.uses(db)
     def download(filename):
-        return downloader(db, settings.UPLOAD_FOLDER, filename) 
+        return downloader(db, settings.UPLOAD_FOLDER, filename)
+
     # To take advantage of this in Form(s)
     # for every field of type upload you MUST specify:
     #
@@ -183,17 +227,15 @@ if settings.UPLOAD_FOLDER:
     # field.download_url = lambda filename: URL('download/%s' % filename)
 
 # #######################################################
-# Optionally configure celery
+# Define and optionally start the scheduler
 # #######################################################
-if settings.USE_CELERY:
-    from celery import Celery
-
-    # to use "from .common import scheduler" and then use it according
-    # to celery docs, examples in tasks.py
-    scheduler = Celery(
-        "apps.%s.tasks" % settings.APP_NAME, broker=settings.CELERY_BROKER
+if settings.USE_SCHEDULER:
+    scheduler = Scheduler(
+        db, logger=logger, max_concurrent_runs=settings.SCHEDULER_MAX_CONCURRENT_RUNS
     )
-
+    scheduler.start()
+else:
+    scheduler = None
 
 # #######################################################
 # Enable authentication
@@ -202,6 +244,10 @@ auth.enable(uses=(session, T, db), env=dict(T=T))
 
 # #######################################################
 # Define convenience decorators
+# They can be used instead of @action and @action.uses
+# They should NEVER BE MIXED with @action and @action.uses
+# If you need to provide extra fixtures for a specific controller
+# add them like this: @authenticated(uses=[extra_fixture])
 # #######################################################
 unauthenticated = ActionFactory(db, session, T, flash, auth)
 authenticated = ActionFactory(db, session, T, flash, auth.user)
