@@ -2,7 +2,7 @@
 
     var mtable = { props: ['url', 'filter', 'order', 'editable', 'create', 'deletable', 'render'], data: null, methods: {}};
     
-    mtable.data = function() {        
+    mtable.data = function() {
         var data = {url: this.url,
                     busy: false,
                     filter: this.filter || '',
@@ -52,10 +52,10 @@
         if (filters.length) url += '&'+filters.join('&');
         if (self.order) url += '&@order='+self.order;
         self.busy = true;
-        axios.get(url).then(function (res) {
+        Q.get(url).then(function (res) {
                 self.busy = false;
-                if(!length) self.table = res.data; 
-                else self.table.items = self.table.items.concat(res.data.items);
+                if(!length) self.table = res.json(); 
+                else self.table.items = self.table.items.concat(res.json().items);
             });
     };
     
@@ -84,28 +84,18 @@
         this.prepare_fields(this.item);
     };
 
-    mtable.methods.l2s = function(list) {
-	return (list||[]).map(function(x){return ''+x}).join(', ');
-    };
-
-    mtable.methods.s2l = function(string, is_list_integer) {
-        var v = string;
-        v = v && v.split(',').map(function(x){return x.trim();}) || [];
-        if (is_list_integer) v = v.map(function(x){return parseInt(x)})
-        return v;
-    };
-
     mtable.methods.prepare_fields = function(item){
         let self = this;
         for(var field of this.table.model){
-            console.log(field.name);
 	    if(field.type == "list:string" || field.type == "list:integer" || field.type.substr(0,14) == "list:reference"){
-		self.string_values[field.name] = mtable.methods.l2s(item[field.name]);
-		console.log(self.string_values[field.name]);
+		self.string_values[field.name] = JSON.stringify(item[field.name]);
 	    } else if (field.type == "datetime") {
-		output = field.default != null ? field.default : '';
-                output = output.split('.')[0];
-		Vue.set(this.item, field.name, output);
+		let value = item[field.name];
+		if (!value) {
+		    value = field.default != null ? field.default : '';
+                    value = value.split('.')[0];
+		    Vue.set(this.item, field.name, value);
+		}
 	    } else if(field.type == "reference"){
                 if (!(field.references in this.reference_options)){
                     Vue.set(this.reference_options, field.references, []);
@@ -113,15 +103,21 @@
                     reference_table_url.pop()
                     reference_table_url.push(field.references)
                     reference_table_url = reference_table_url.join('/') + '?@options_list=true';
-                    axios.get(reference_table_url).then(function (res) {
-                        let url_components = res.config.url.split('?')[0].split('/');
-                        self.reference_options[url_components[url_components.length - 1 ]] = res.data.items;
+                    console.log(reference_table_url);
+                    Q.get(reference_table_url).then(function (res) {
+                        let url_components = res.url.split('?')[0].split('/');
+                        self.reference_options[url_components[url_components.length - 1 ]] = res.json().items;
                      });
                     
                 }
             }
         }
-    }
+	this.$nextTick(function(){
+            Q("input[type=text].type-list-string,input[type=text].type-list-integer,input[type=text].type-list-reference").forEach(
+               function(elem){Q.tags_input(elem);
+            });
+	});
+    };
 
     mtable.methods.parse_and_validate_json = function(event){
         try {
@@ -137,7 +133,7 @@
         if (window.confirm("Really delete record?")) {
             let url = this.url + '/' + item.id;            
             this.table.items = this.table.items.filter((i)=>{return i.id != item.id;});
-            axios.delete(url);
+            Q.delete(url);
             if (item==this.item) this.item = null;
         }
     };
@@ -148,33 +144,40 @@
 	for(var field of this.table.model) {
 	    var is_list_integer = field.type == "list:integer" || field.type.substr(0,14) == "list:reference";
 	    if(field.type == "list:string" || is_list_integer) {
-		item[field.name] = mtable.methods.s2l(this.string_values[field.name], is_list_integer);
+		try {
+		    item[field.name] = JSON.parse(this.string_values[field.name]);
+		} catch(err) {
+		    alert("Invalid field value: " + field.name);
+		    break;
+		}
 	    }
 	}
         if (item.id) {
             url += '/' + item.id;
-            axios.put(url, item).then(mtable.handle_response('put', this),
-                                      mtable.handle_response('put', this));
+	    var data = JSON.parse(JSON.stringify(item));
+	    delete data["id"];
+            Q.put(url, data).then(mtable.handle_response('put', this),
+                                  mtable.handle_response('put', this));
         } else {
-            axios.post(url, item).then(mtable.handle_response('post', this),
-                                       mtable.handle_response('post', this));
-        }
+            Q.post(url, item).then(mtable.handle_response('post', this),
+                                   mtable.handle_response('post', this));
+        }	
     };
 
     mtable.handle_response = function(method, data) {
         self.busy = false;
         return function(res) {
-            if (res.response) res = res.response; // deal with error weirdness
+	    res = res.json();
             if (method == 'post') {
                 data.table.items = [];
-                console.log(data);
                 mtable.methods.load.call(data);
             }
-            if (res.data.status == 'success') {
+            if (res.status == 'success') {
                 data.clear();
+		location.reload();
             } else {
-                data.errors = res.data.errors;
-                data.message = res.data.message;
+                data.errors = res.errors;
+                data.message = res.message;
             }
         };
     };
@@ -207,13 +210,11 @@
         window.location = window.location.href.split('?')[0]+'?'+source;
     };
 
-
-
     var scripts = document.getElementsByTagName('script');
     var src = scripts[scripts.length-1].src;
     var path = src.substr(0, src.length-3) + '.html';
-    Q.register_vue_component('mtable', path, function(template) {        
-            mtable.template = template.data;
-            return mtable;
-        });
+    Q.register_vue_component('mtable', path, function(template) {
+        mtable.template = template.data;
+        return mtable;
+    });
 })();
