@@ -1,6 +1,5 @@
 import sys
 
-
 # Note, when changing the Highlight.js css,
 # the background color of .file .diff should match the
 # background in the .hljs class in gitlog.min.css
@@ -87,7 +86,6 @@ def escape(txt):
 # specify which language we want to render as for highlight.js
 # based on the name of the file
 def getFileType(name):
-
     if name.lower().endswith(".py"):
         return 'class="language-python"'
     if name.lower().endswith(".js"):
@@ -163,7 +161,7 @@ def diff2kryten(data):
         + '</style></head><body><div style="text-align:right">'
         + "</div>"
         + div
-        + '<script src="/_dashboard/static/js/jquery.min.js"></script>'
+        + '<script src="https://unpkg.com/jquery.min.js"></script>'
         + '<script src="/_dashboard/static/js/highlight.min.js"></script>'
         + "<script>"
         + (script % block)
