@@ -3,7 +3,6 @@ import smtplib
 
 
 class EmailAuthPlugin:
-
     name = "email_auth"
     label = "Email"
 
@@ -40,11 +39,12 @@ class EmailAuthPlugin:
             server.login(email, password)
             server.quit()
             return True
-        except:
+        except Exception:
             logging.exception("email_auth() failed")
             if server:
                 try:
                     server.quit()
-                except:  # server might already close connection after error
+                except Exception:
+                    # server might already close connection after error
                     pass
             return False
