@@ -15,7 +15,6 @@ ldap.set_option(ldap.OPT_REFERRALS, 0)
 
 
 class LDAPPlugin(UsernamePassword):
-
     name = "ldap"
     label = "LDAP"
 
@@ -70,9 +69,10 @@ class LDAPPlugin(UsernamePassword):
             base_dn='ou=Users,dc=domain,dc=com',
             tls=True))
 
-    If you need to bind to the directory with an admin account in order to
-    search it then specify bind_dn & bind_pw to use for this.
-    - currently only implemented for Active Directory
+    If you need to bind to the directory with a bind account in order to
+    search it, then specify bind_dn & bind_pw to use for this.
+    - currently only implemented for Active Directory where anonymous bind
+    is normally not allowed.
 
     If you need to restrict the set of allowed users (e.g. to members of a
     department) then specify an rfc4515 search filter string.
@@ -89,7 +89,12 @@ class LDAPPlugin(UsernamePassword):
            ))
 
     Where:
-    manage_user - let web2py handle user data from ldap
+    manage_user: bool
+        If True py4web will fetch and update user profile
+        fields (first name, last name, email) from LDAP/AD on each login and
+        keep them in sync with its db.
+        If False, only authentication is performed and user profile fields
+        are taken only from py4web db.
     user_firstname_attrib - the attribute containing the user's first name
                             optionally you can specify parts.
                             Example: cn: "John Smith" - 'cn:1'='John'
@@ -113,7 +118,7 @@ class LDAPPlugin(UsernamePassword):
            ))
 
         Where:
-        manage_groups - let web2py handle the groups from ldap
+        manage_groups - let py4web handle the groups from ldap
         db - is the database object (need to have auth_user, auth_group,
             auth_membership)
         group_dn - the ldap branch of the groups
@@ -141,10 +146,8 @@ class LDAPPlugin(UsernamePassword):
         group_filterstr - as the filterstr but for group select
 
     If using Active Directory you must specify bind_dn and bind_pw for
-    allowed_groups unless anonymous bind works.
+    allowed_groups because anonymous bind is not normally allowed.
 
-    You can set the logging level with the "logging_level" parameter, default
-    is "error" and can be set to error, warning, info, debug.
     """
 
     def __init__(
@@ -182,7 +185,6 @@ class LDAPPlugin(UsernamePassword):
         logger=logging,
         groups=None,
     ):
-
         self.server = server
         self.port = port
         self.base_dn = base_dn
@@ -219,45 +221,28 @@ class LDAPPlugin(UsernamePassword):
         self.groups = groups
 
     def check_credentials(self, username, password):
-
-        server = self.server
-        port = self.port
         base_dn = self.base_dn
         mode = self.mode
-        secure = self.secure
-        self_signed_certificate = self.self_signed_certificate
-        cert_path = self.cert_path
-        cert_file = self.cert_file
-        cacert_path = self.cacert_path
-        cacert_file = self.cacert_file
-        key_file = self.key_file
         bind_dn = self.bind_dn
         bind_pw = self.bind_pw
         filterstr = self.filterstr
         username_attrib = self.username_attrib
         custom_scope = self.custom_scope
-        allowed_groups = self.allowed_groups
         manage_user = self.manage_user
         user_firstname_attrib = self.user_firstname_attrib
         user_lastname_attrib = self.user_lastname_attrib
         user_mail_attrib = self.user_mail_attrib
         manage_groups = self.manage_groups
-        manage_groups_callback = self.manage_groups_callback
         db = self.db
-        group_dn = self.db
-        group_name_attrib = self.group_name_attrib
-        group_member_attrib = self.group_member_attrib
-        group_filterstr = self.group_filterstr
         group_mapping = self.group_mapping
-        tls = self.tls
         logger = self.logger
 
         if password == "":  # http://tools.ietf.org/html/rfc4513#section-5.1.2
             logger.warning("blank password not allowed")
             return False
         logger.debug(
-            "mode: [%s] manage_user: [%s] custom_scope: [%s] manage_groups: [%s]"
-            % (str(mode), str(manage_user), str(custom_scope), str(manage_groups))
+            f"mode: {str(mode)}, manage_user: {str(manage_user)}, \
+            custom_scope: {str(custom_scope)}, manage_groups: {str(manage_groups)}"
         )
         if manage_user:
             if user_firstname_attrib.count(":") > 0:
@@ -294,7 +279,7 @@ class LDAPPlugin(UsernamePassword):
                     for x in base_dn.split(","):
                         if "DC=" in x.upper():
                             domain.append(x.split("=")[-1])
-                    username = "%s@%s" % (username, ".".join(domain))
+                    username = f"{username}@{'.'.join(domain)}"
                 username_bare = username.split("@")[0]
                 con.set_option(ldap.OPT_PROTOCOL_VERSION, 3)
                 # In cases where ForestDnsZones and DomainDnsZones are found,
@@ -318,14 +303,14 @@ class LDAPPlugin(UsernamePassword):
                 result = con.search_ext_s(
                     base_dn,
                     ldap.SCOPE_SUBTREE,
-                    "(&(sAMAccountName=%s)(%s))"
-                    % (ldap.filter.escape_filter_chars(username_bare), filterstr),
+                    f"(&(sAMAccountName={ldap.filter.escape_filter_chars(username_bare)})({filterstr}))",
                     requested_attrs,
                 )[0][1]
+                logger.info(f"Login result: {result}")
                 if not isinstance(result, dict):
                     # result should be a dict in the form
                     # {'sAMAccountName': [username_bare]}
-                    logger.warning("User [%s] not found!" % username)
+                    logger.warning(f"User {username} not found!")
                     return False
                 if bind_dn:
                     # We know the user exists & is in the correct OU
@@ -366,7 +351,7 @@ class LDAPPlugin(UsernamePassword):
                     con.simple_bind_s(bind_dn, bind_pw)
                     dn = "uid=" + username + "," + base_dn
                     dn = con.search_s(
-                        base_dn, ldap.SCOPE_SUBTREE, "(uid=%s)" % username, [""]
+                        base_dn, ldap.SCOPE_SUBTREE, f"(uid={username, ['']})"
                     )[0][0]
                 else:
                     dn = "uid=" + username + "," + base_dn
@@ -386,10 +371,7 @@ class LDAPPlugin(UsernamePassword):
                 # bind anonymously
                 con.simple_bind_s(dn, pw)
                 # search by e-mail address
-                filter = "(&(mail=%s)(%s))" % (
-                    ldap.filter.escape_filter_chars(username),
-                    filterstr,
-                )
+                filter = f"(&(mail={ldap.filter.escape_filter_chars(username)})({filterstr}))"
                 # find the uid
                 attrs = ["uid"]
                 if manage_user:
@@ -411,9 +393,8 @@ class LDAPPlugin(UsernamePassword):
                     basedns = base_dn
                 else:
                     basedns = [base_dn]
-                filter = "(&(uid=%s)(%s))" % (
-                    ldap.filter.escape_filter_chars(username),
-                    filterstr,
+                filter = (
+                    f"(&(uid={ldap.filter.escape_filter_chars(username)})({filterstr}))"
                 )
                 found = False
                 for basedn in basedns:
@@ -425,14 +406,13 @@ class LDAPPlugin(UsernamePassword):
                             con.simple_bind_s(user_dn, password)
                             found = True
                             break
-                    except ldap.LDAPError as detail:
+                    except ldap.LDAPError:
                         (exc_type, exc_value) = sys.exc_info()[:2]
                         logger.warning(
-                            "ldap_auth: searching %s for %s resulted in %s: %s\n"
-                            % (basedn, filter, exc_type, exc_value)
+                            f"ldap_auth: searching {basedn} for {filter} resulted in {exc_type}: {exc_value}\n"
                         )
                 if not found:
-                    logger.warning("User [%s] not found!" % username)
+                    logger.warning(f"User [{username}] not found!")
                     return False
                 result = result[0][1]
             if mode == "custom":
@@ -442,11 +422,7 @@ class LDAPPlugin(UsernamePassword):
                     basedns = base_dn
                 else:
                     basedns = [base_dn]
-                filter = "(&(%s=%s)(%s))" % (
-                    username_attrib,
-                    ldap.filter.escape_filter_chars(username),
-                    filterstr,
-                )
+                filter = f"(&({username_attrib}={ldap.filter.escape_filter_chars(username)})({filterstr}))"
                 if custom_scope == "subtree":
                     scope = ldap.SCOPE_SUBTREE
                 elif custom_scope == "base":
@@ -463,18 +439,17 @@ class LDAPPlugin(UsernamePassword):
                             con.simple_bind_s(user_dn, password)
                             found = True
                             break
-                    except ldap.LDAPError as detail:
+                    except ldap.LDAPError:
                         (exc_type, exc_value) = sys.exc_info()[:2]
                         logger.warning(
-                            "ldap_auth: searching %s for %s resulted in %s: %s\n"
-                            % (basedn, filter, exc_type, exc_value)
+                            f"ldap_auth: searching {basedn} for {filter} resulted in {exc_type}: {exc_value}\n"
                         )
                 if not found:
-                    logger.warning("User [%s] not found!" % username)
+                    logger.warning(f"User {username} not found!")
                     return False
                 result = result[0][1]
             if manage_user:
-                logger.info("[%s] Manage user data" % str(username))
+                logger.info(f"[{str(username)}] Manage user data")
                 try:
                     store_sso_id = "ldap:" + username
                     user_firstname = result[user_firstname_attrib][0]
@@ -484,7 +459,7 @@ class LDAPPlugin(UsernamePassword):
                         )[user_firstname_part]
                     else:
                         store_user_firstname = user_firstname
-                except KeyError as e:
+                except KeyError:
                     store_user_firstname = None
                 try:
                     user_lastname = result[user_lastname_attrib][0]
@@ -494,11 +469,11 @@ class LDAPPlugin(UsernamePassword):
                         )[user_lastname_part]
                     else:
                         store_user_lastname = user_lastname
-                except KeyError as e:
+                except KeyError:
                     store_user_lastname = None
                 try:
                     store_user_mail = result[user_mail_attrib][0]
-                except KeyError as e:
+                except KeyError:
                     store_user_mail = None
                 update_or_insert_values = {
                     "first_name": store_user_firstname,
@@ -540,55 +515,24 @@ class LDAPPlugin(UsernamePassword):
 
             con.unbind()
             return True
-        except ldap.INVALID_CREDENTIALS as e:
+        except ldap.INVALID_CREDENTIALS:
             return False
-        except ldap.LDAPError as e:
+        except ldap.LDAPError:
             import traceback
 
-            logger.warning("[%s] Error in ldap processing" % str(username))
+            logger.warning(f"[{str(username)}] Error in Ldap processing")
             logger.debug(traceback.format_exc())
             print(traceback.format_exc())
             return False
-        except IndexError as ex:  # for AD membership test
+        except IndexError:  # for AD membership test
             import traceback
 
-            logger.warning("[%s] Ldap result indexing error" % str(username))
+            logger.warning(f"[{str(username)}] Ldap result indexing error")
             logger.debug(traceback.format_exc())
             return False
 
     def is_user_in_allowed_groups(self, username, password=None):
-
-        server = self.server
-        port = self.port
-        base_dn = self.base_dn
-        mode = self.mode
-        secure = self.secure
-        self_signed_certificate = self.self_signed_certificate
-        cert_path = self.cert_path
-        cert_file = self.cert_file
-        cacert_path = self.cacert_path
-        cacert_file = self.cacert_file
-        key_file = self.key_file
-        bind_dn = self.bind_dn
-        bind_pw = self.bind_pw
-        filterstr = self.filterstr
-        username_attrib = self.username_attrib
-        custom_scope = self.custom_scope
         allowed_groups = self.allowed_groups
-        manage_user = self.manage_user
-        user_firstname_attrib = self.user_firstname_attrib
-        user_lastname_attrib = self.user_lastname_attrib
-        user_mail_attrib = self.user_mail_attrib
-        manage_groups = self.manage_groups
-        manage_groups_callback = self.manage_groups_callback
-        db = self.db
-        group_dn = self.db
-        group_name_attrib = self.group_name_attrib
-        group_member_attrib = self.group_member_attrib
-        group_filterstr = self.group_filterstr
-        group_mapping = self.group_mapping
-        tls = self.tls
-        logger = self.logger
 
         """
         Figure out if the username is a member of an allowed group
@@ -617,40 +561,13 @@ class LDAPPlugin(UsernamePassword):
         ones in py4web's application database or create new groups
         according to ldap.
         """
-        server = self.server
-        port = self.port
-        base_dn = self.base_dn
-        mode = self.mode
-        secure = self.secure
-        self_signed_certificate = self.self_signed_certificate
-        cert_path = self.cert_path
-        cert_file = self.cert_file
-        cacert_path = self.cacert_path
-        cacert_file = self.cacert_file
-        key_file = self.key_file
-        bind_dn = self.bind_dn
-        bind_pw = self.bind_pw
-        filterstr = self.filterstr
-        username_attrib = self.username_attrib
-        custom_scope = self.custom_scope
-        allowed_groups = self.allowed_groups
-        manage_user = self.manage_user
-        user_firstname_attrib = self.user_firstname_attrib
-        user_lastname_attrib = self.user_lastname_attrib
-        user_mail_attrib = self.user_mail_attrib
-        manage_groups = self.manage_groups
         manage_groups_callback = self.manage_groups_callback
         db = self.db
-        group_dn = self.db
-        group_name_attrib = self.group_name_attrib
-        group_member_attrib = self.group_member_attrib
-        group_filterstr = self.group_filterstr
         group_mapping = self.group_mapping
-        tls = self.tls
         logger = self.logger
         groups = self.groups
 
-        logger.info("[%s] Manage user groups" % str(username))
+        logger.info(f"[{str(username)}] Manage user groups")
         try:
             #
             # Get all group name where the user is in actually in ldap
@@ -663,7 +580,7 @@ class LDAPPlugin(UsernamePassword):
                     if group in group_mapping:
                         l.append(group_mapping[group])
                 ldap_groups_of_the_user = l
-                logger.info("User groups after remapping: %s" % str(l))
+                logger.info(f"User groups after remapping: {str(l)}")
 
             #
             # Get all group name where the user is in actually in local db
@@ -683,7 +600,7 @@ class LDAPPlugin(UsernamePassword):
                         .first()
                         .id
                     )
-                except AttributeError as e:
+                except AttributeError:
                     #
                     # There is no user in local db
                     # We create one
@@ -692,12 +609,12 @@ class LDAPPlugin(UsernamePassword):
                         db_user_id = db.auth_user.insert(
                             username=username, first_name=username
                         )
-                    except AttributeError as e:
+                    except AttributeError:
                         db_user_id = db.auth_user.insert(
                             email=username, first_name=username
                         )
             if not db_user_id:
-                logger.error("There is no username or email for %s!" % username)
+                logger.error(f"There is no username or email for {username}!")
                 raise
             db_groups_of_the_user = groups.get(db_user_id)
 
@@ -722,7 +639,7 @@ class LDAPPlugin(UsernamePassword):
                 for callback in manage_groups_callback:
                     callback()
         except:
-            logger.warning("[%s] Groups are not managed successfully!" % str(username))
+            logger.warning(f"[{str(username)}] Groups are not managed successfully!")
             import traceback
 
             logger.debug(traceback.format_exc())
@@ -731,46 +648,26 @@ class LDAPPlugin(UsernamePassword):
 
     def _init_ldap(self):
         """
-        Inicialize ldap connection
+        Inizialize ldap connection
         """
 
         server = self.server
         port = self.port
-        base_dn = self.base_dn
-        mode = self.mode
         secure = self.secure
         self_signed_certificate = self.self_signed_certificate
-        cert_path = self.cert_path
         cert_file = self.cert_file
         cacert_path = self.cacert_path
         cacert_file = self.cacert_file
         key_file = self.key_file
-        bind_dn = self.bind_dn
-        bind_pw = self.bind_pw
-        filterstr = self.filterstr
-        username_attrib = self.username_attrib
-        custom_scope = self.custom_scope
-        allowed_groups = self.allowed_groups
-        manage_user = self.manage_user
-        user_firstname_attrib = self.user_firstname_attrib
-        user_lastname_attrib = self.user_lastname_attrib
-        user_mail_attrib = self.user_mail_attrib
-        manage_groups = self.manage_groups
-        manage_groups_callback = self.manage_groups_callback
-        db = self.db
-        group_dn = self.db
-        group_name_attrib = self.group_name_attrib
-        group_member_attrib = self.group_member_attrib
-        group_filterstr = self.group_filterstr
-        group_mapping = self.group_mapping
         tls = self.tls
         logger = self.logger
 
-        logger.info("[%s] Initialize ldap connection" % str(server))
+        logger.debug(f"[{str(server)}] Initialize ldap connection")
         if secure:
             if not port:
                 port = 636
 
+            logger.debug(f"Using Secure LDAP connection to {server}:{port}")
             if self_signed_certificate:
                 # NOTE : If you have a self-signed SSL Certificate pointing over "port=686" and "secure=True" alone
                 #        will not work, you need also to set "self_signed_certificate=True".
@@ -793,50 +690,32 @@ class LDAPPlugin(UsernamePassword):
         else:
             if not port:
                 port = 389
-            con = ldap.initialize("ldap://" + server + ":" + str(port))
+            logger.debug(f"Initializing LDAP connection to ldap://{server}:{str(port)}")
+            con = ldap.initialize(f"ldap://{server}:{str(port)}")
         if tls:
             con.start_tls_s()
         return con
 
-    def get_user_groups_from_ldap(self, con, username):
+    def get_user_groups_from_ldap(self, username=None, password=None):
         """
         Get all group names from ldap where the user is in
         """
-
-        server = self.server
-        port = self.port
-        base_dn = self.base_dn
-        mode = self.mode
-        secure = self.secure
-        self_signed_certificate = self.self_signed_certificate
-        cert_path = self.cert_path
-        cert_file = self.cert_file
-        cacert_path = self.cacert_path
-        cacert_file = self.cacert_file
-        key_file = self.key_file
         bind_dn = self.bind_dn
         bind_pw = self.bind_pw
+        base_dn = self.base_dn
+        mode = self.mode
         filterstr = self.filterstr
-        username_attrib = self.username_attrib
-        custom_scope = self.custom_scope
-        allowed_groups = self.allowed_groups
-        manage_user = self.manage_user
-        user_firstname_attrib = self.user_firstname_attrib
-        user_lastname_attrib = self.user_lastname_attrib
-        user_mail_attrib = self.user_mail_attrib
-        manage_groups = self.manage_groups
-        manage_groups_callback = self.manage_groups_callback
-        db = self.db
         group_dn = self.group_dn
         group_name_attrib = self.group_name_attrib
         group_member_attrib = self.group_member_attrib
         group_filterstr = self.group_filterstr
-        group_mapping = self.group_mapping
-        tls = self.tls
         logger = self.logger
-        groups = self.groups
 
-        logger.info("[%s] Get user groups from ldap" % str(username))
+        # if username is None, return empty list
+        if username is None:
+            return []
+
+        logger.info(f"[{str(username)}] Get user groups from ldap")
         #
         # Get all group name where the user is in actually in ldap
         # #########################################################
@@ -849,9 +728,19 @@ class LDAPPlugin(UsernamePassword):
                 for x in base_dn.split(","):
                     if "DC=" in x.upper():
                         domain.append(x.split("=")[-1])
-                username = "%s@%s" % (username, ".".join(domain))
+                username = f"{username}@{'.'.join(domain)}"
             username_bare = username.split("@")[0]
+            con = self._init_ldap()
             con.set_option(ldap.OPT_PROTOCOL_VERSION, 3)
+            try:
+                if bind_dn:
+                    # need to search directory with an admin account 1st
+                    con.simple_bind_s(bind_dn, bind_pw)
+                else:
+                    # credentials should be in the form of user...@domain.tld
+                    con.simple_bind_s(username, password)
+            except (ldap.INVALID_CREDENTIALS, ldap.OPERATIONS_ERROR):
+                return []
             # In cases where ForestDnsZones and DomainDnsZones are found,
             # result will look like the following:
             # ['ldap://ForestDnsZones.domain.com/DC=ForestDnsZones,
@@ -861,7 +750,7 @@ class LDAPPlugin(UsernamePassword):
             username = con.search_ext_s(
                 base_dn,
                 ldap.SCOPE_SUBTREE,
-                "(&(sAMAccountName=%s)(%s))" % (bare, filterstr),
+                f"(&(sAMAccountName={bare})({filterstr}))",
                 ["cn"],
             )[0][0]
 
@@ -869,11 +758,9 @@ class LDAPPlugin(UsernamePassword):
         if username is None:
             return []
         # search for groups where user is in
-        filter = "(&(%s=%s)(%s))" % (
-            ldap.filter.escape_filter_chars(group_member_attrib),
-            ldap.filter.escape_filter_chars(username),
-            group_filterstr,
-        )
+        filter = f"(&({ldap.filter.escape_filter_chars(group_member_attrib)}=\
+            {ldap.filter.escape_filter_chars(username)})({group_filterstr}))"
+
         group_search_result = con.search_s(
             group_dn, ldap.SCOPE_SUBTREE, filter, [group_name_attrib]
         )
@@ -884,14 +771,6 @@ class LDAPPlugin(UsernamePassword):
                 ldap_groups_of_the_user.append(
                     str(group[group_name_attrib][0], encoding="utf-8")
                 )
-                print(ldap_groups_of_the_user)
 
-        logger.debug("User groups: %s" % ldap_groups_of_the_user)
+        logger.debug(f"User groups for {username}: {ldap_groups_of_the_user}")
         return list(ldap_groups_of_the_user)
-
-        if filterstr[0] == "(" and filterstr[-1] == ")":  # rfc4515 syntax
-            filterstr = filterstr[1:-1]  # parens added again where used
-        return []
-
-    def check_credentials(self, username, password):
-        return self.is_user_in_allowed_groups(username, password)
