@@ -1,7 +1,5 @@
 # from https://requests-oauthlib.readthedocs.io/en/latest/examples/github.html
-import requests
 
-from py4web import URL
 
 from . import OAuth2
 
@@ -19,4 +17,3 @@ class OAuth2Github(OAuth2):
         "email": "email",
         "sso_id": "url",
     }
-
