@@ -8,10 +8,10 @@
 """
 
 import email.utils
-import json
 import logging
 import mimetypes
 import os
+import re
 import smtplib
 from email import message_from_string
 from email.encoders import encode_base64
@@ -189,23 +189,18 @@ class Mailer:
                 if filename is None:
                     raise Exception("Missing attachment name")
                 payload = payload.read()
-            # FIXME PY3 can be used to_native?
-            filename = filename.encode(encoding)
             if content_type is None:
-                content_type = mimetypes.guess_type(filename)
+                content_type = mimetypes.guess_type(filename)[0]
             self.my_filename = filename
             self.my_payload = payload
             MIMEBase.__init__(self, *content_type.split("/", 1))
             self.set_payload(payload)
-            self["Content-Disposition"] = 'attachment; filename="%s"' % to_native(
-                filename, encoding
-            )
+            self["Content-Disposition"] = f'attachment; filename="{filename}"'
             if content_id is not None:
-                self["Content-Id"] = "<%s>" % to_native(content_id, encoding)
+                self["Content-Id"] = f"<{content_id}>"
             Encoders.encode_base64(self)
 
     def __init__(self, server=None, sender=None, login=None, tls=True, ssl=False):
-
         settings = self.settings = Settings()
         settings.logger = logging
         settings.server = server
@@ -425,7 +420,6 @@ class Mailer:
                 html = None
 
         if (text is not None or html is not None) and (not raw):
-
             if text is not None:
                 if not isinstance(text, basestring):
                     text = text.read()
@@ -509,8 +503,6 @@ class Mailer:
             #                   sign                   #
             ############################################
             if sign:
-                import string
-
                 core.check_version(None)
                 pin = payload_in.as_string().replace("\n", "\r\n")
                 plain = core.Data(pin)
@@ -534,7 +526,7 @@ class Mailer:
                         "signed",
                         boundary=None,
                         _subparts=None,
-                        **dict(micalg="pgp-sha1", protocol="application/pgp-signature")
+                        **dict(micalg="pgp-sha1", protocol="application/pgp-signature"),
                     )
                     # Insert the origin payload
                     payload.attach(payload_in)
@@ -578,7 +570,7 @@ class Mailer:
                         "encrypted",
                         boundary=None,
                         _subparts=None,
-                        **dict(protocol="application/pgp-encrypted")
+                        **dict(protocol="application/pgp-encrypted"),
                     )
                     p = MIMEBase("application", "pgp-encrypted")
                     p.set_payload("Version: 1\r\n")
@@ -741,16 +733,16 @@ class Mailer:
             to.extend(bcc)
         payload["Subject"] = encoded_or_raw(to_unicode(subject, encoding))
         payload["Date"] = email.utils.formatdate()
-        for k, v in iteritems(headers):
+        for k, v in headers.items():
             payload[k] = encoded_or_raw(to_unicode(v, encoding))
 
         list_unsubscribe = list_unsubscribe or self.settings.list_unsubscribe
         if list_unsubscribe:
-            payload['List-Unsubscribe'] = "<mailto:%s>" % list_unsubscribe
+            payload["List-Unsubscribe"] = "<mailto:%s>" % list_unsubscribe
 
         dkim = dkim or self.settings.dkim
         if dkim:
-            payload['DKIM-Signature'] = dkim_sign(payload, dkim.key, dkim.selector)
+            payload["DKIM-Signature"] = dkim_sign(payload, dkim.key, dkim.selector)
 
         result = {}
         try:
@@ -792,7 +784,7 @@ class Mailer:
                         body=to_unicode(text or "", encoding),
                         html=html,
                         attachments=attachments,
-                        **xcc
+                        **xcc,
                     )
                 elif html and (not raw):
                     result = google_mail.send_mail(
@@ -801,7 +793,7 @@ class Mailer:
                         subject=to_unicode(subject, encoding),
                         body=to_unicode(text or "", encoding),
                         html=html,
-                        **xcc
+                        **xcc,
                     )
                 else:
                     result = google_mail.send_mail(
@@ -809,7 +801,7 @@ class Mailer:
                         to=origTo,
                         subject=to_unicode(subject, encoding),
                         body=to_unicode(text or "", encoding),
-                        **xcc
+                        **xcc,
                     )
             elif self.settings.server == "aws" and boto3:
                 client = boto3.client("ses")
@@ -818,8 +810,9 @@ class Mailer:
                     response = client.send_raw_email(
                         RawMessage=raw, Source=sender, Destinations=to
                     )
+                    print("Message send:", response)
                     return True
-                except ClientError as e:
+                except ClientError:
                     raise RuntimeError()
             else:
                 smtp_args = self.settings.server.split(":")
@@ -833,17 +826,19 @@ class Mailer:
                         server.ehlo(self.settings.hostname)
                     if self.settings.login:
                         server.login(*self.settings.login.split(":", 1))
+                    if "Message-ID" not in payload:
+                        payload["Message-ID"] = email.utils.make_msgid()
                     result = server.sendmail(sender, to, payload.as_string())
                 finally:
                     # do not want to hide errors raising some exception here
                     try:
                         server.quit()
-                    except:
+                    except Exception:
                         pass
                     # ensure to close any socket with SMTP server
                     try:
                         server.close()
-                    except:
+                    except Exception:
                         pass
         except Exception as e:
             self.settings.logger.warning("Mailer.send failure:%s" % e)
@@ -855,31 +850,35 @@ class Mailer:
 
 
 def dkim_sign(payload, dkim_key, dkim_selector):
-
     import dkim
 
     # sign all existing mail headers except those specified in
     # http://dkim.org/specs/rfc4871-dkimbase.html#rfc.section.5.5
-    headers = list(filter(
-        lambda h: h not in [
-            "Return-Path",
-            "Received",
-            "Comments",
-            "Keywords",
-            "Resent-Bcc",
-            "Bcc",
-            "DKIM-Signature",
-        ],
-        payload))
+    headers = list(
+        filter(
+            lambda h: h
+            not in [
+                "Return-Path",
+                "Received",
+                "Comments",
+                "Keywords",
+                "Resent-Bcc",
+                "Bcc",
+                "DKIM-Signature",
+            ],
+            payload,
+        )
+    )
 
     domain = re.sub(r".*@", "", payload["From"])
     domain = re.sub(r">.*", "", domain)
 
     sig = dkim.sign(
-            message=payload.as_bytes(),
-            selector=dkim_selector.encode(),
-            domain=domain.encode(),
-            privkey=dkim_key.encode(),
-            include_headers=[h.encode() for h in headers])
+        message=payload.as_bytes(),
+        selector=dkim_selector.encode(),
+        domain=domain.encode(),
+        privkey=dkim_key.encode(),
+        include_headers=[h.encode() for h in headers],
+    )
 
-    return sig[len("DKIM-Signature: "):].decode()
+    return sig[len("DKIM-Signature: ") :].decode()
