@@ -1,6 +1,7 @@
 import base64
 import hashlib
 import json
+import os
 import time
 import uuid
 
@@ -49,8 +50,8 @@ class URLVerifier(Fixture):
             if self.url_signer.lifespan is not None:
                 if float(ts) + self.url_signer.lifespan < time.time():
                     raise HTTP(403)
-        except:
-            raise HTTP(403)
+        except Exception as err:
+            raise HTTP(403, body=str(err))
 
     def _decode_ts(self, ts_string):
         """Decodes the timestamp, removing the salt."""
@@ -59,7 +60,6 @@ class URLVerifier(Fixture):
 
 
 class URLSigner(Fixture):
-
     RESERVED_VARIABLES = ["_ts", "_signature"]
 
     def __init__(
@@ -115,10 +115,6 @@ class URLSigner(Fixture):
         assert "_signature" not in self.variables_to_sign
         self.algo = algo or hashlib.sha256
 
-    @property
-    def local(self):
-        return self._safe_local        
-
     def on_request(self, context):
         """Creates the signing key if necessary."""
         if self.session is not None and self.session.get("_signature_key") is None:
@@ -158,6 +154,9 @@ class URLSigner(Fixture):
         h = self.algo(self.get_key())
         ts = "%.3f" % time.time()
         salt = str(uuid.uuid1())
+        url_prefix = os.environ.get("PY4WEB_URL_PREFIX", "")
+        if url_prefix:
+            url = url_prefix + url
         h.update(self.get_info_to_sign(url, variables, ts, salt))
         sig = base64.b16encode(h.digest()).decode("utf-8")
         sig_content = json.dumps(dict(ts=ts, salt=salt, sig=sig))
